@@ -93,15 +93,16 @@
  */
 
 #include <stdarg.h>
-#include <fcntl.h>
+#include <linux/types.h>
+#include <linux/fcntl.h>
 #ifdef IBCS_COMPATIBLE
 #include <ibcs/unistd.h>
 #else
 #include <linux/unistd.h>
 #endif
 #include <linux/elf.h>
-#include <sys/mman.h>
-#include <link.h>
+#include <linux/mman.h>
+#include "link.h"
 
 #include "sysdep.h"
 #include "hash.h"
@@ -204,6 +205,7 @@ void _dl_boot(int args){
   struct r_debug * debug_addr;
   unsigned int *chains;
   int indx;
+  int _dl_secure;
 
   /* First obtain the information on the stack that tells us more about
      what binary is loaded, where it is loaded, etc, etc */
@@ -516,14 +518,15 @@ void _dl_boot(int args){
 	 (dl_data[AT_UID] != -1 && dl_data[AT_UID] == dl_data[AT_EUID] &&
 	  dl_data[AT_GID] == dl_data[AT_EGID]))
     {
+      _dl_secure = 0;
       _dl_preload = _dl_getenv("LD_PRELOAD", envp);
       _dl_library_path = _dl_getenv("LD_LIBRARY_PATH",envp);
     }
     else
     {
-      _dl_unsetenv("LD_PRELOAD", envp);
+      _dl_secure = 1;
+      _dl_preload = _dl_getenv("LD_PRELOAD", envp);
       _dl_unsetenv("LD_AOUT_PRELOAD", envp);
-      _dl_preload = NULL;
       _dl_unsetenv("LD_LIBRARY_PATH", envp);
       _dl_unsetenv("LD_AOUT_LIBRARY_PATH", envp);
       _dl_library_path = NULL;
@@ -546,35 +549,35 @@ void _dl_boot(int args){
     struct elf_resolve *tpnt1;
     char *lpnt;
 
-    if (_dl_preload)  {
-      int i;
-      char *str;
-      char c = _dl_preload[0];
-      str = _dl_preload; 
-      for (i = 0; c != 0; i++) {
-	if (_dl_preload[i] == ':' ||
-	    _dl_preload[i] == '\0') {
-	  c= _dl_preload[i];
-	  _dl_preload[i]=0;
+    if (_dl_preload) {
+      char c, *str, *str2;
+
+      str = _dl_preload;
+      while (*str == ':' || *str == ' ' || *str == '\t')
+	str++;
+      while (*str) {
+	str2 = str;
+	while (*str2 && *str2 != ':' && *str2 != ' ' && *str2 != '\t')
+	  str2++;
+	c = *str2;
+	*str2 = '\0';
+	if (!_dl_secure || _dl_strchr(str, '/') == NULL) {
 	  tpnt1 = _dl_load_shared_library(NULL, str);
 	  if (!tpnt1) {
 	    if (_dl_trace_loaded_objects)
 	      _dl_fdprintf(1, "\t%s => not found\n", str);
-	    else
-	    {
+	    else {
 	      _dl_fdprintf(2, "%s: can't load library '%s'\n",
 			   _dl_progname, str);
 	      _dl_exit(15);
 	    }
-	  }
-	  else
-	  {
-	    if (_dl_trace_loaded_objects && !tpnt1->usage_count)
-	    {
+	  } else {
+	    if (_dl_trace_loaded_objects && !tpnt1->usage_count) {
 	      /* this is a real hack to make ldd not print the
 		 library itself when run on a library. */
 	      if (_dl_strcmp(_dl_progname, str) != 0)
-		_dl_fdprintf(1, "\t%s => %s\n", str, tpnt1->libname);
+		_dl_fdprintf(1, "\t%s => %s (0x%x)\n", str, tpnt1->libname,
+			     (unsigned)tpnt1->loadaddr);
 	    }
 	    rpnt->next = 
 	      (struct dyn_elf *) _dl_malloc(sizeof(struct dyn_elf));
@@ -585,9 +588,11 @@ void _dl_boot(int args){
 	    tpnt1->libtype = elf_lib;
 	    rpnt->dyn = tpnt1;
 	  }
-	  str=_dl_preload+i+1;
-	  _dl_preload[i]=c;
 	}
+	*str2 = c;
+	str = str2;
+	while (*str == ':' || *str == ' ' || *str == '\t')
+	  str++;
       }
     }
 
@@ -646,7 +651,8 @@ void _dl_boot(int args){
 		}
 	      } else {
 		if (_dl_trace_loaded_objects && !tpnt1->usage_count)
-		  _dl_fdprintf(1, "\t%s => %s\n", cp2, tpnt1->libname);
+		  _dl_fdprintf(1, "\t%s => %s (0x%x)\n", cp2, tpnt1->libname,
+			       (unsigned)tpnt1->loadaddr);
                 rpnt->next = 
 		  (struct dyn_elf *) _dl_malloc(sizeof(struct dyn_elf));
 		_dl_memset (rpnt->next, 0, sizeof (*(rpnt->next)));
@@ -711,7 +717,8 @@ void _dl_boot(int args){
 	  else
 	  {
 	    if (_dl_trace_loaded_objects && !tpnt1->usage_count)
-	      _dl_fdprintf(1, "\t%s => %s\n", lpnt, tpnt1->libname);
+	      _dl_fdprintf(1, "\t%s => %s (0x%x)\n", lpnt, tpnt1->libname,
+			   (unsigned)tpnt1->loadaddr);
 	    rpnt->next = 
 	      (struct dyn_elf *) _dl_malloc(sizeof(struct dyn_elf));
 	    _dl_memset (rpnt->next, 0, sizeof (*(rpnt->next)));
@@ -734,7 +741,7 @@ void _dl_boot(int args){
   if(_dl_trace_loaded_objects)
     {
       _dl_warn = _dl_getenv("LD_WARN", envp);
-      if(!_dl_warn) _dl_exit(0);
+      if (!_dl_warn) _dl_exit(0);
     }
 
   /*
