@@ -5,9 +5,9 @@
  */
 
 #include "dlfcn.h"
-#include <link.h>
+#include "link.h"
 #include <stdlib.h>
-#include <sys/mman.h>
+#include <linux/mman.h>
 #ifdef IBCS_COMPATIBLE
 #include <ibcs/unistd.h>
 #else
