@@ -21,11 +21,12 @@
 /* This file contains the helper routines to load an ELF sharable
    library into memory and add the symbol table info to the chain. */
 
-#include <fcntl.h>
-#include <errno.h>
-#include <string.h>
-#include <stdlib.h>
-#include <sys/mman.h>
+#include <linux/types.h>
+#include <linux/fcntl.h>
+#include <linux/errno.h>
+#include "string.h"
+/*#include <stdlib.h>*/
+#include <linux/mman.h>
 #include "hash.h"
 #include "linuxelf.h"
 #include "sysdep.h"
@@ -35,7 +36,6 @@
 #include <linux/unistd.h>
 #endif
 #include "syscall.h"
-#include "string.h"
 #ifdef USE_CACHE
 #include "../config.h"
 #endif
@@ -143,7 +143,7 @@ struct elf_resolve * _dl_load_shared_library(struct elf_resolve * tpnt,
 
   _dl_internal_error_number = 0;
 
-  /* quick hack to ensure mylibname buffer doesn't overflow.  don't
+  /* quick hack to ensure mylibname buffer doesn't overflow.  don't 
      allow full_libname or any directory to be longer than 1024. */
   if (_dl_strlen(full_libname) > 1024)
     goto goof;
@@ -191,14 +191,14 @@ struct elf_resolve * _dl_load_shared_library(struct elf_resolve * tpnt,
 	pnt1 += (unsigned int) tpnt->loadaddr + tpnt->dynamic_info[DT_STRTAB];
 	while(*pnt1){
 	  pnt2 = mylibname;
-          while(*pnt1 && *pnt1 != ':') {
-            if (pnt2 - mylibname < 1024)
-              *pnt2++ = *pnt1++;
-            else
-              pnt1++;
-          }
-          if (pnt2 - mylibname >= 1024)
-            break;
+	  while(*pnt1 && *pnt1 != ':') {
+	    if (pnt2 - mylibname < 1024)
+	      *pnt2++ = *pnt1++;
+	    else
+	      pnt1++;
+	  }
+	  if (pnt2 - mylibname >= 1024)
+	    break;
 	  if(pnt2[-1] != '/') *pnt2++ = '/';
 	  pnt = libname;
 	  while(*pnt) *pnt2++  = *pnt++;
@@ -218,13 +218,13 @@ struct elf_resolve * _dl_load_shared_library(struct elf_resolve * tpnt,
     while (*pnt1) {
       pnt2 = mylibname;
       while(*pnt1 && *pnt1 != ':' && *pnt1 != ';') {
-        if (pnt2 - mylibname < 1024)
-          *pnt2++ = *pnt1++;
-       else
-         pnt1++;
+	if (pnt2 - mylibname < 1024)
+	  *pnt2++ = *pnt1++;
+	else
+	  pnt1++;
       }
       if (pnt2 - mylibname >= 1024)
-        break;
+	break;
       if(pnt2[-1] != '/') *pnt2++ = '/';
       pnt = libname;
       while(*pnt) *pnt2++  = *pnt++;
@@ -251,7 +251,8 @@ struct elf_resolve * _dl_load_shared_library(struct elf_resolve * tpnt,
 
     for (i = 0; i < header->nlibs; i++)
     {
-      if (libent[i].flags == LIB_ELF &&
+      if ((libent[i].flags == LIB_ELF || 
+	   libent[i].flags == LIB_ELF_LIBC5) &&
 	  _dl_strcmp(libname, strs+libent[i].sooffset) == 0 &&
 	  (tpnt1 = _dl_load_elf_shared_library(strs+libent[i].liboffset, 0)))
 	return tpnt1;
