@@ -14,7 +14,9 @@ extern inline int _dl_open(char * addr, unsigned int flags);
 extern inline int _dl_write(int fd, const char * buf, int len);
 extern inline int _dl_read(int fd, const char * buf, int len);
 extern inline int _dl_mprotect(const char * addr, int size, int prot);
-#include <sys/stat.h>
+#define new_stat stat
+#include <asm/stat.h>
+#undef new_stat
 extern inline int _dl_stat(char * filename, struct stat *st);
 extern inline int _dl_munmap(char * addr, int size);
 
