@@ -45,7 +45,9 @@ __res; })
 #define do_div(n,base) _dl_div ((n)/(base))
 #endif
 
-static char * number(char * str, long num, int base, int size, int precision
+#define ADD_CHAR(s,n,c) ( ((n) > 1) ? *(s)++ = (c), (n)-- : (c) )
+
+static char * number(char * str, int *bufsize, long num, int base, int size, int precision
 	,int type)
 {
 	char c,sign,tmp[66];
@@ -89,25 +91,25 @@ static char * number(char * str, long num, int base, int size, int precision
 	size -= precision;
 	if (!(type&(ZEROPAD+LEFT)))
 		while(size-->0)
-			*str++ = ' ';
+			ADD_CHAR(str, *bufsize, ' ');
 	if (sign)
-		*str++ = sign;
+		ADD_CHAR(str, *bufsize, sign);
 	if (type & SPECIAL)
 		if (base==8)
-			*str++ = '0';
+			ADD_CHAR(str, *bufsize, '0');
 		else if (base==16) {
-			*str++ = '0';
-			*str++ = digits[33];
+			ADD_CHAR(str, *bufsize, '0');
+			ADD_CHAR(str, *bufsize, digits[33]);
 		}
 	if (!(type & LEFT))
 		while (size-- > 0)
-			*str++ = c;
+			ADD_CHAR(str, *bufsize, c);
 	while (i < precision--)
-		*str++ = '0';
+		ADD_CHAR(str, *bufsize, '0');
 	while (i-- > 0)
-		*str++ = tmp[i];
+		ADD_CHAR(str, *bufsize, tmp[i]);
 	while (size-- > 0)
-		*str++ = ' ';
+		ADD_CHAR(str, *bufsize, ' ');
 	return str;
 }
 
@@ -126,14 +128,15 @@ int _dl_fdprintf(int fd, const char *fmt, ...)
 				   number of chars for from string */
 	int qualifier;		/* 'h', 'l', or 'L' for integer fields */
 
-	char buf[1024];
+	int bufsize;
+	char buf[2048];
 	va_list(args);
 
 	va_start(args, fmt);
 
-	for (str=buf ; *fmt ; ++fmt) {
+	for (str=buf, bufsize=sizeof buf ; *fmt ; ++fmt) {
 		if (*fmt != '%') {
-			*str++ = *fmt;
+			ADD_CHAR(str, bufsize, *fmt);
 			continue;
 		}
 			
@@ -192,10 +195,10 @@ int _dl_fdprintf(int fd, const char *fmt, ...)
 		case 'c':
 			if (!(flags & LEFT))
 				while (--field_width > 0)
-					*str++ = ' ';
-			*str++ = (unsigned char) va_arg(args, int);
+					ADD_CHAR(str, bufsize, ' ');
+			ADD_CHAR(str, bufsize, (unsigned char) va_arg(args, int));
 			while (--field_width > 0)
-				*str++ = ' ';
+				ADD_CHAR(str, bufsize, ' ');
 			continue;
 
 		case 's':
@@ -207,11 +210,11 @@ int _dl_fdprintf(int fd, const char *fmt, ...)
 
 			if (!(flags & LEFT))
 				while (len < field_width--)
-					*str++ = ' ';
+					ADD_CHAR(str, bufsize, ' ');
 			for (i = 0; i < len; ++i)
-				*str++ = *s++;
+			        ADD_CHAR(str, bufsize, *s++);
 			while (len < field_width--)
-				*str++ = ' ';
+				ADD_CHAR(str, bufsize, ' ');
 			continue;
 
 		case 'p':
@@ -219,7 +222,7 @@ int _dl_fdprintf(int fd, const char *fmt, ...)
 				field_width = 2*sizeof(void *);
 				flags |= ZEROPAD;
 			}
-			str = number(str,
+			str = number(str, &bufsize,
 				(unsigned long) va_arg(args, void *), 16,
 				field_width, precision, flags);
 			continue;
@@ -254,9 +257,9 @@ int _dl_fdprintf(int fd, const char *fmt, ...)
 
 		default:
 			if (*fmt != '%')
-				*str++ = '%';
+				ADD_CHAR(str, bufsize, '%');
 			if (*fmt)
-				*str++ = *fmt;
+				ADD_CHAR(str, bufsize, *fmt);
 			else
 				--fmt;
 			continue;
@@ -272,7 +275,7 @@ int _dl_fdprintf(int fd, const char *fmt, ...)
 			num = va_arg(args, int);
 		else
 			num = va_arg(args, unsigned int);
-		str = number(str, num, base, field_width, precision, flags);
+		str = number(str, &bufsize, num, base, field_width, precision, flags);
 	}
 	*str = '\0';
 	_dl_write(fd, buf, str-buf);
