@@ -18,7 +18,7 @@ Cambridge, MA 02139, USA.  */
 
 #include <ansidecl.h>
 #include <stdio.h>
-#include <linux/string.h>
+#include <string.h>
 
 #ifndef HAVE_GNU_LD
 #define _sys_errlist sys_errlist
@@ -44,9 +44,9 @@ DEFUN(strerror, (errnum), int errnum)
       static char unknown_error[] = "Unknown error 000000000000000000000";
       static char fmt[] = "Unknown error %d";
 #ifdef __linux__
-      sprintf(unknown_error, fmt, errnum);
+      snprintf(unknown_error, sizeof unknown_error, fmt, errnum);
 #else
-      size_t len = sprintf(unknown_error, fmt, errnum);
+      size_t len = snprintf(unknown_error, sizeof unknown_error, fmt, errnum);
       if (len < sizeof(fmt) - 2)
 	return NULL;
       unknown_error[len] = '\0';
