@@ -5,26 +5,49 @@
 #include <sys/stat.h>
 #include <sys/mman.h>
 #include <linux/elf.h>
+#include "../config.h"
+#include <sys/types.h>
 
 #define ELF32_R_SYM(x) ((x) >> 8)
 #define ELF32_R_TYPE(x) ((x) & 0xff)
 
+void warn(char *fmt, ...);
 char *xstrdup(char *);
 
-char *readsoname(FILE *infile)
+struct needed_tab
+{
+  char *soname;
+  int type;
+};
+
+struct needed_tab needed_tab[] = {
+  { "libc.so.5",    LIB_ELF_LIBC5 },
+  { "libm.so.5",    LIB_ELF_LIBC5 },
+  { "libdl.so.1",   LIB_ELF_LIBC5 },
+  { "libc.so.6",    LIB_ELF_LIBC6 },
+  { "libm.so.6",    LIB_ELF_LIBC6 },
+  { "libdl.so.2",   LIB_ELF_LIBC6 },
+  { NULL,           LIB_ELF }
+};
+
+char *readsoname(char *name, FILE *infile, int *type)
 {
   struct elfhdr *epnt;
   struct elf_phdr *ppnt;
-  int i;
+  int i, j;
   char *header;
   unsigned int dynamic_addr = 0;
   unsigned int dynamic_size = 0;
   int strtab_val = 0;
-  int soname_val = 0;
+  int needed_val;
   int loadaddr = -1;
   struct dynamic *dpnt;
   struct stat st;
-  char *res = NULL;
+  char *needed;
+  char *soname = NULL;
+  int multi_libcs = 0;
+
+  *type = LIB_ELF;
 
   if (fstat(fileno(infile), &st))
     return NULL;
@@ -64,22 +87,45 @@ char *readsoname(FILE *infile)
   {
     if (dpnt->d_tag == DT_STRTAB)
       strtab_val = dpnt->d_un.d_val;
-    if (dpnt->d_tag == DT_SONAME)
-      soname_val = dpnt->d_un.d_val;
     dpnt++;
   };
 
-  if (!strtab_val || !soname_val)
-    goto skip;
-  if (soname_val + strtab_val - loadaddr < 0 ||
-      soname_val + strtab_val - loadaddr >= st.st_size)
+  if (!strtab_val)
     goto skip;
 
-  res = xstrdup((char *) (header - loadaddr + strtab_val + soname_val));
+  dpnt = (struct dynamic *) &header[dynamic_addr];
+  for (i = 0; i < dynamic_size; i++)
+  {
+    if (dpnt->d_tag == DT_SONAME || dpnt->d_tag == DT_NEEDED)
+    {
+      needed_val = dpnt->d_un.d_val;
+      if (needed_val + strtab_val - loadaddr >= 0 ||
+	  needed_val + strtab_val - loadaddr < st.st_size)
+      {
+	needed = (char *) (header - loadaddr + strtab_val + needed_val);
+
+	if (dpnt->d_tag == DT_SONAME)
+	  soname = xstrdup(needed);
+
+	for (j = 0; needed_tab[j].soname != NULL; j++)
+	{
+	  if (strcmp(needed, needed_tab[j].soname) == 0)
+	  {
+	    if (*type != LIB_ELF && *type != needed_tab[j].type)
+	      multi_libcs = 1;
+	    *type = needed_tab[j].type;
+	  }
+	}
+      }
+    }
+    dpnt++;
+  };
+  
+  if (multi_libcs)
+    warn("%s appears to be for multiple libc's", name);
 
  skip:
   munmap(header, st.st_size);
 
-  return res;
+  return soname;
 }
-
